import json
from typing import Any
from fastapi import WebSocket


class ConnectionManager:
    def __init__(self):
        self.active_connections: list[WebSocket] = []

    async def connect(self, websocket: WebSocket):
        await websocket.accept()
        self.active_connections.append(websocket)

    def disconnect(self, websocket: WebSocket):
        self.active_connections.remove(websocket)

    async def send_personal_message(self, message: str, websocket: WebSocket):
        await websocket.send_text(message)

    async def broadcast(self, type: str, data: Any):
        for connection in self.active_connections:
            await connection.send_text(
                json.dumps(
                    {
                        "type": type,
                        "data": data,
                    }
                )
            )


manager = ConnectionManager()
